#!/usr/local/bin/perl

# Explode the MIME attachments of the messages we have been passed
# and store them all ready for virus scanning.
# Return a hash of message ids --> MIME entities.

use strict;
use MIME::Parser;

my($cp) = "/usr/bin/cp";

sub ExplodeMessages {
  my($QDir, $DestDir, @Ids) = @_;

  local(*PIPE);
  my($id, $path, $header, $parser, $entity, %entities);

  foreach $id (@Ids) {
    $path = "$DestDir/$id";
    $header = "$DestDir/$id.header";

    $parser = new MIME::Parser;
    $parser->output_dir($path);
    $parser->output_to_core('NONE');
    unless (open(PIPE, "cat $header $QDir/df$id |")) {
      Log::WarnLog("Cannot cat $header and $QDir/df$id, %m");
      next;
    }
    unless ($entity = $parser->read(\*PIPE)) {
      close PIPE;
      Log::WarnLog("Cannot parse $header and $QDir/df$id, %m");
      next;
    }

    close PIPE;
    $entities{$id} =  $entity;
  }
  %entities;
}

# Print the skeleton of all the messages
sub DumpSkeletons {
  my($entities) = @_;
  my($entity, $id);

  while(($id, $entity) = each %$entities) {
    print "\nMessage $id looks like this\n";
    print "---------------------------\n";
    $entity->dump_skeleton;
  }
}

# Replace 1 infected section with clean text file attachment.
sub DisinfectEntity {
  my($infected, $parent, $filename, $report) = @_;

  my($i, @parts, $infectednum, $Warning);

  @parts = $parent->parts;
  # Find the infected part
  $infectednum = -1;
  for ($i=0; $i<@parts; $i++) {
    ($infectednum=$i),last if $parts[$i]==$infected;
  }
  Log::WarnLog("Oh shit, missed infected entity in message :-( %m"), return
    if $infectednum<0;

  # Now to actually do something about it...
  $Warning = ConstructWarning($filename, $report);
  $parts[$infectednum] = build MIME::Entity
                           Type => 'text/plain',
                           Filename => 'VirusWarning.txt',
                           Disposition => 'attachment',
                           Data => $Warning,
                           Encoding => 'quoted-printable',
                           Charset => 'us-ascii',
                           Top => 0;
  $parent->parts(\@parts);
}

# Disinfect all the infected entities
sub Disinfect {
  my($Reports, $File2Entity, $Entity2Parent, $Entity2File) = @_;

  my($id,$parts,$file,$text,$entity);

  while(($id, $parts) = each %$Reports) {
    while(($file, $text) = each %$parts) {
      $entity = $File2Entity->{$id}{$file};
      DisinfectEntity($entity,
                      $Entity2Parent->{$entity},
                      $Entity2File->{$entity},
                      $text);
    }
  }
}

# Construct an infection warning message to the user based on the filename
sub ConstructWarning {
  my($file, $report) = @_;
  my($date) = scalar localtime;
  return <<EOW;
This is a message from the ECS E-Mail Virus Protection Service
--------------------------------------------------------------
Your original e-mail attachment "$file"
was infected by a virus and has been replaced by this warning message.

If you wish to receive a copy of the *infected* attachment, please
e-mail helpdesk\@ecs.soton.ac.uk and include the whole of this message
in your request.

At $date the virus scanner said:
   $report
Note to Help Desk: Look on scanner in /opt/virus/var/quarantine.
--
postmaster\@ecs.soton.ac.uk
EOW
}

# Write out all the messages to text files
sub WriteMessages {
  my($Dir, $IdList, $entities) = @_;
  my($message);
  local(*ID);

  chdir $Dir or do {
    Log::WarnLog("Cannot chdir to $Dir to write messages, %m");
    return;
  };

  foreach $message (@$IdList) {
    print "Writing $Dir/$message\n";
    open(ID, ">$message") or next;
    $entities->{$message}->print_header(\*ID);
    print ID "\n";
    $entities->{$message}->print_body(\*ID);
    close ID;
  }
}

# Save the infected attachments into the quarantine area.
sub QuarantineInfections {
  my($Reports, $QuarDir, $InDir) = @_;
  my($day,$month,$year);
  my($TodayDir, $MsgDir);
  my($id,$parts,$dirtyfile);

  # Construct the quarantine dir for this message
  mkdir($QuarDir, 0700) unless -d $QuarDir;
  ($day,$month,$year) = (localtime)[3,4,5];
  $month++;
  $year += 1900;
  $TodayDir = "$QuarDir/" . sprintf("%02d%02d%04d", $day, $month, $year);
  mkdir($TodayDir, 0700) unless -d $TodayDir;

  while(($id,$parts) = each %$Reports) {
    $MsgDir = "$TodayDir/$id";
    mkdir($MsgDir, 0700) unless -d $MsgDir;
    foreach $dirtyfile (keys %$parts) {
      # Write the infected file into this dir
      Log::InfoLog("Saved infections to $MsgDir");
      system("$cp -p \"$InDir/$id/$dirtyfile\" \"$MsgDir/$dirtyfile\"");
    }
  }

  chdir $QuarDir;
}

1;
