#!/usr/local/bin/perl

# Main program....

use strict;
use POSIX;

require 'logger.pl';
require 'Sendmail.pl';
require 'Workarea.pl';
require 'explode.pl';
require 'sweep.pl';
require 'info.pl';
require 'config.pl';

my($ConfigFile) = '/opt/virus/etc/virus.conf';

# Globals set by config.pl
use vars qw($Debugging $MaxCleanPerRun $MaxDirtyPerRun
            $MaxCleanBytes $MaxDirtyBytes
            $InQueueDir  $OutQueueDir $PidFile $SrcDir $DestDir
            $QuarantineDir $Sendmail);

# Start Syslog-ing
Log::Start();
ReadConfig($ConfigFile);

# Become a daemon and save our PID
unless ($Debugging) {
  fork && exit;
  setsid();
}
local(*PID);
open(PID, ">$PidFile") or Log::DieLog("Cannot write pid file $PidFile, %m");
print PID "$$\n";
close PID;
undef *PID;

# Clean up the entire outgoing sendmail queue in case I was
# killed off half way through processing some messages.
# Read the entire incoming mail 
my($Msgs, $Bytes);
my @MessagesIn = Sendmail::ListMessages($InQueueDir, \$Msgs, \$Bytes);
Sendmail::ClearOutQueue(\@MessagesIn, $OutQueueDir);

while(1) {
  # Clear up the work area
  ClearWorkArea($SrcDir);

  # Find all the messages to work on
  my(@DefinitelyClean, @MessagesIn);
  Sendmail::FindMessagesToProcess($InQueueDir, $SrcDir,
                                  \@DefinitelyClean, \@MessagesIn,
                                  $MaxCleanPerRun, $MaxDirtyPerRun,
                                  $MaxCleanBytes, $MaxDirtyBytes);

  # Immediately deliver messages I don't need to scan
  Sendmail::DeliverCleanIds(\@DefinitelyClean, 'unused',
                            $InQueueDir, $OutQueueDir, $Sendmail)
    if @DefinitelyClean;

  # Wait and go round again if nothing to scan
  sleep(30), next unless @MessagesIn;

  # Construct directory hierarchy for the message attachments
  BuildInDirs($SrcDir, @MessagesIn);

  # Write the headers so they look like real email message headers
  #Sendmail::WriteHeaderFiles($InQueueDir, $SrcDir, @MessagesIn);

  # Find all the messages we can guarantee to be clean, and deliver
  # them straight out to the users.
  #my $DefinitelyClean = Sendmail::FindCleanMessages($SrcDir, \@MessagesIn);
  #print "Definitely clean messages are " . join(', ', @$DefinitelyClean) . "\n";
  #Sendmail::DeliverCleanIds($DefinitelyClean, 'unused',
  #                          $InQueueDir, $OutQueueDir, $Sendmail);

  # Save all the attachments into files
  my(%MimeEntities);
  %MimeEntities = ExplodeMessages($InQueueDir, $SrcDir, @MessagesIn);
  #DumpSkeletons(\%MimeEntities);

  # Check all the attachments for viruses and work out
  # the cleanliness of each message.
  # $InfectionReports is a ref to a hash of hashes (id,filename)
  my($InfectionReports, @CleanIds, @DirtyIds, $Start, $Duration);
  $Start = time;
  $InfectionReports = Sweep::VirusScan($SrcDir);
  $Duration = time - $Start;
  Log::InfoLog("Scanned $Msgs messages, $Bytes bytes in $Duration seconds");
  Sweep::CleanAndDirty($InfectionReports, \@MessagesIn,
                       \@CleanIds, \@DirtyIds);

  # Debugging shit...
  Sweep::PrintSummary($InfectionReports, \@CleanIds, \@DirtyIds);

  my(%NumParts, $File2Entity, %Entity2Parent, %Entity2File);
  $File2Entity = EntitiesInfo(\%MimeEntities, \%NumParts,
                              \%Entity2Parent, \%Entity2File);

  # More debugging...
  #PrintParts(\%NumParts);
  #PrintFilenames($File2Entity);
  #PrintInfectedSections($InfectionReports, $File2Entity);
  #PrintParents(\%Entity2Parent);

  # Deliver all the clean messages straight into the outgoing queue
  #Sendmail::Deliver($InQueueDir, $OutQueueDir, $OutSendmail, @CleanIds);

  # Replace all the infected attachments with a text file
  # note: use parts arrayref for this.
  Disinfect($InfectionReports, $File2Entity, \%Entity2Parent, \%Entity2File);

  #print "\n\nAfter disinfection...\n" . '='x21 . "\n";
  #DumpSkeletons(\%MimeEntities);

  # Deliver all the cleaned up messages into the outgoing queue and read
  # the original recipient list for each one in the process
  #WriteMessages($DestDir, \@MessagesIn, \%MimeEntities);
  Sendmail::DeliverCleanIds(\@CleanIds, \%MimeEntities,
                            $InQueueDir, $OutQueueDir, $Sendmail);
  Sendmail::DeliverIds(\@DirtyIds, \%MimeEntities,
                       $InQueueDir, $OutQueueDir, $Sendmail);

  # Quarantine the infected attachments
  QuarantineInfections($InfectionReports, $QuarantineDir, $SrcDir);

  # Notify the original recipients of each infected message about the problem
  # Nah... Sod 'em...

  last if $Debugging;
}
