#!/usr/local/bin/perl

# Run Sophos sweep over the directory hierarchy I'm given.
# Parse the output and find all the viruses.
# Passed in a directory..
# Return a hash. Keys are "messageid,messagepart", values are reports
# and only exist when there was a virus in the messagepart.

use strict;
package Sweep;

my($sweep) = '/opt/sophos/bin/sophoswrapper';
# In next line, '-ss' makes it work nice and quietly
my($options) = '-sc -f -all -rec -ss -archive -loopback --no-follow-symlinks --no-reset-atime';

sub PrintSummary {
  my($Reports, $Clean, $Dirty) = @_;
  my($id,$part,$value,$text);

  while(($id,$value) = each %$Reports) {
    while(($part, $text) = each %$value) {
      print "Virus found in message $id part $part\n   $text\n";
    }
  }
  print "Clean Messages are " . join(', ', @$Clean) . "\n";
  print "Dirty Messages are " . join(', ', @$Dirty) . "\n";
}

# Construct lists of the clean and infected messages.
# Passed in a hash of reports and a ref to list of message ids.
# Pass out a list of clean message ids and a list of infected message ids.
sub CleanAndDirty {
  my($Reports, $IdList, $Clean, $Dirty) = @_;
  my(%clean, %dirty, $key, $id, $part);

  map { $clean{$_} = 1 } @$IdList;
  foreach $id (keys %$Reports) {
    #($id,$part) = split(/,/, $key);
    delete $clean{$id};
    $dirty{$id} = 1;
  }
  @$Clean = keys %clean;
  @$Dirty = keys %dirty;
}

sub VirusScan {
  my($BaseDir) = @_;

  my(%infections, $NumInfections);

  $NumInfections = CallSophos($BaseDir, \%infections);
  $NumInfections += CallOwnChecking($BaseDir, \%infections);

  Log::InfoLog("Found $NumInfections viruses in messages " .
          join(',', keys %infections)) if $NumInfections;

  return \%infections;
}

sub CallSophos {
  my($BaseDir, $infections) = @_;

  my($infected, $dot, $id, $part, $report, $NumInfections);
  local(*SWEEP);

  chdir $BaseDir or die "Cannot chdir to $BaseDir, $!";

  open(SWEEP, "$sweep $options . |") or die "Cannot run sweep, $!";
  while(<SWEEP>) {
    chomp;
    print "$_\n";
    Log::WarnLog($_) if /error/i;
    next unless /virus.*found/i;
    $report = $_;
    $infected = $_;
    $infected =~ s/^.*found\s*in\s*file\s*//i;
    ($dot, $id, $part) = split(/\//, $infected, 3);
    $infections->{"$id"}{"$part"} = $report;
    $NumInfections++;
  }
  close SWEEP; # Check the return code here
  Log::InfoLog("Sweep returned $?") if $?;
  return $NumInfections;
}

sub CallOwnChecking {
  my($BaseDir, $infections);

  # Insert your own checking here.
  # In $BaseDir, you will find a directory for each message, which has the
  # same name as the message id. Also there is a messageid.header file
  # containing all the headers for the message.
  # Add entries into %$infections, where they are referenced as
  # $infections->{"message id"}{"filename"} but please don't over-write ones
  # that are already there.
  # Return the number of infections/problems you found.

  return 0;
}

1;
